"""
    created 28.06.2018 by Jens Diemer <opensource@jensdiemer.de>
    :copyleft: 2018 by the django-for-runners team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""
from for_runners.models.discipline import DisciplineModel
from for_runners.models.distance import DistanceModel
from for_runners.models.event import EventLinkModel, EventModel, ParticipationModel, CostModel
from for_runners.models.gpx import GpxModel
