{% extends "admin/change_list.html" %}
{% load i18n admin_urls %}

{% block footer %}<div id="footer">
 <footer>
  <p>
      Django-ForRunners links:
      <a href="http://github.com/jedie/django-for-runners">GitHub</a>
      |
      <a href="https://pypi.org/project/django-for-runners/">PyPi</a>
  </p>
</footer>
</div>{% endblock %}