from __future__ import absolute_import

from .version import __version__  # noqa

# https://docs.djangoproject.com/en/2.0/ref/applications/#configuring-applications-ref
default_app_config = "for_runners.apps.ForRunnersConfig"
