# Generated by Django 2.0.7 on 2018-07-31 18:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('for_runners', '0004_auto_20180719_1739'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='gpxmodel',
            name='net_duration',
        ),
        migrations.AlterField(
            model_name='gpxmodel',
            name='duration',
            field=models.PositiveIntegerField(blank=True, help_text='Duration in seconds (from the GPX track)', null=True),
        ),
        migrations.AlterField(
            model_name='gpxmodel',
            name='length',
            field=models.PositiveIntegerField(blank=True, help_text='Length in meters (calculated from GPX track 3-dimensional used latitude, longitude, and elevation)', null=True),
        ),
        migrations.RenameField(
            model_name='gpxmodel',
            old_name='duration',
            new_name='duration_s',
        ),
        migrations.AlterField(
            model_name='gpxmodel',
            name='duration_s',
            field=models.PositiveIntegerField(blank=True, editable=False, help_text='Duration in seconds (from the GPX track)', null=True),
        ),
    ]
