{% load i18n admin_urls static admin_list %}

<noscript><strong>Please enable JavaScript!</strong></noscript>
<canvas id="chart" style="width: 100%; height: 500px;"></canvas>

<script src="/static/chartjs/Chart.bundle.min.js"></script>

<script>{% include "for_runners/includes/chartjs_distance_pace_statistics.js" %}</script>



